/*
 * getentry.c
 * 
 * Kevin P. Smith  1/31/89
 * 
 * Get an outfit and team entry request from user.
 */
#include "copyright.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "packets.h"
#ifndef _NO_DEPEND
#   include "ntserv_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern int		      shutdown P_((int s, int how));
extern int                    fprintf P_((FILE *, const char *, ...));
extern int                    fclose P_((FILE *));
extern int                    printf P_((const char *, ...));
extern int                    fflush P_((FILE *));
extern pid_t                  getpid P_((void));
extern int                    close P_((int));

#undef P_
#endif

extern int      overload;

void
getEntry(team, stype)
   int            *team;
   int            *stype;
{
   int             i;
   int             switching = -1;	/* team change -- TC */
   char            ibuf[80];

   inputMask = CP_OUTFIT;
   for (;;) {
      /* updateShips so he knows how many players on each team */
      updateShips(1);
      sendMaskPacket(tournamentMask(me->p_team));
      flushSockBuf();
#ifdef nodef
      /* Have we been busted? */
      if (me->p_status == PFREE) {
	 me->p_status = PDEAD;
	 me->p_explode = 600/UPDATE_NORM;
      }
#endif	/* nodef */
      /*
       * the problem with the above statement is in the following scenario:
       * Intially OUTFITTIME is at 6 minutes.  Once OUTFITTIME expires
       * me->p_status is set to PFREE.  If another connection comes in it
       * will grab this slot.  At this point, p_status == PDEAD.  Now when
       * p_explode expires 60 seconds later p_status is again set to PFREE --
       * slot problem again. And so on..  By simply setting ghostbuster to 0
       * here we avoid the problem all together.
       */

      me->p_ghostbuster = 0;
      socketPause();
      readFromClient();
      if (isClientDead()) {
	 int             i;

	 if (noressurect)
	    exitGame(0);
#ifdef DEBUG
	 printf("Ack!  The client went away!\n");
	 printf("I will attempt to resurrect him!\n");
	 fflush(stdout);
#endif
	 /* UDP fail-safe */
	 commMode = COMM_TCP;
	 if (udpSock >= 0)
	    closeUdpConn();
	 /* For next two minutes, we try to restore connection */
	 shutdown(sock, 2);
	 sock = -1;
	 for (i = 0;; i++) {
	    switch (me->p_status) {
	    case PFREE:
	       me->p_status = PDEAD;
	       me->p_explode = 600/UPDATE_NORM;
	       break;
	    case PALIVE:
	       me->p_ghostbuster = 0;
	       break;
	    case PDEAD:
	       me->p_explode = 600/UPDATE_NORM;
	       break;
	    default:
	       me->p_explode = 600/UPDATE_NORM;
	       me->p_ghostbuster = 0;
	       break;
	    }
	    my_sleep(5);
	    if (connectToClient(host, nextSocket))
	       break;
	    if (i == 23) {
#ifdef DEBUG
	       printf("Oh well, maybe I'm getting rusty!\n");
	       fflush(stdout);
#endif
	       switch (me->p_status) {
	       case PFREE:
		  break;
	       case PALIVE:
		  me->p_ghostbuster = 1000000;
		  break;
	       case PDEAD:
		  me->p_explode = 0;
		  break;
	       default:
		  me->p_explode = 0;
		  me->p_ghostbuster = 1000000;
		  break;
	       }
	       exitGame(1);
	    }
	 }
#ifdef DEBUG
	 printf("A miracle!  He's alive!\n");
	 fflush(stdout);
#endif
#ifdef DS
	 if(me->p_process != getpid()){
	    /* whoops, co-pilot.  We're outa here */
	    forceShutdown(-10);
	 }
#endif
	 teamPick = -1;
	 updateSelf();
	 updateShips(1);
	 flushSockBuf();
      }
      if (teamPick != -1) {
	 if (teamPick < 0 || teamPick > 3) {
	    warning(1, "Get real!", -1);
	    sendPickokPacket(0);
	    teamPick = -1;
	    continue;
	 }
	 if (!(tournamentMask(me->p_team) & (1 << teamPick))) {
	    if (gamedown || !status->gameup)
	       warning(1, "Sorry, the server is down.", -1);
	    else
	       warning(1, "I cannot allow that.  Pick another team", 9);
	    sendPickokPacket(0);
	    teamPick = -1;
	    continue;
	 }
	 /* confirm team change -- TC */
	 if (((1 << teamPick) != me->p_team) &&
	     (me->p_team != ALLTEAM)){	/* switching teams 7/27/91 TC */
	    if ((switching != teamPick) && (me->p_whydead != KGENOCIDE)) {
	       switching = teamPick;
	       warning(1, "Please confirm change of teams.  Select the new team again.", 10);
	       sendPickokPacket(0);
	       teamPick = -1;
	       continue;
	    }
	 }
	 /* His team choice is ok. */
	 if (shipPick == ATT) {
	    if (strcmp(me->p_name, nombreDios) == 0) {
	       FILE           *fp;

	       fp = fopen(LOGFILENAME, "a");
	       if (fp) {
		  fprintf(fp, "God entering (%s).\n", me->p_monitor);
		  fclose(fp);
	       }
	       break;
	    } else {
	       FILE           *fp;

	       fp = fopen(LOGFILENAME, "a");
	       if (fp) {
		  fprintf(fp, "God attempted (%s).\n", me->p_monitor);
		  fprintf(fp, "nombreDios=%s\n", nombreDios);
		  fclose(fp);
	       }
	       warning(1, "Are you a God?", -1);
	       sendPickokPacket(0);
	       teamPick = -1;
	       continue;
	    }
	 }
	 if (shipPick < 0 || shipPick >= ATT) {
	    warning(1, "That is an illegal ship type.  Try again.", 11);
	    sendPickokPacket(0);
	    teamPick = -1;
	    continue;
	 }
	 if (shipsallowed[shipPick] == 0) {
	    /* if ((shipsallowed[shipPick]==0) && (shipPick != ATT)) */
	    warning(1, "That ship hasn't been designed yet.  Try again.", -1);
	    sendPickokPacket(0);
	    teamPick = -1;
	    continue;
	 }
	 if (shipPick == STARBASE) {
	    if (teams[1 << teamPick].s_turns > 0 && !chaos) {
	       int             min;
	       /*
	        * tell how much time remains til starbase is ready. note that
	        * PLANETFUSE determines when s_turns is decremented It is
	        * currently set at 400 -- 20 minutes for reconstruction
	        */
	       /*
	        * update is in micro-secs -- convert to sec and force round
	        * up XX - this will wrap if UPDATE becomes greater then
	        * 400000
	        */
	       min = ((UPDATE * teams[1 << teamPick].s_turns * 400) / 1000000 + 30)
		  / 60;

	       sprintf(ibuf,
	       "Your new starbase requires %d more %s of construction time.",
		       min, min > 1 ? "minutes" : "minute");
	       warning(1, ibuf, -1);
	       sendPickokPacket(0);
	       teamPick = -1;
	       continue;
	    }
	    if (mystats->st_rank < sbrank) {
	       char            buf[100];

	       sprintf(buf, "You need a rank of %s or higher to command a starbase!", ranks[sbrank].name);
	       warning(1, buf, -1);
	       sendPickokPacket(0);
	       teamPick = -1;
	       continue;
	    }
	    if (numShips(1 << teamPick) < 0 && !chaos) {
	       warning(1, "Your team is not capable of defending such an expensive ship!", -1);
	       sendPickokPacket(0);
	       teamPick = -1;
	       continue;
	    }
	    if (numPlanets(1 << teamPick) < 5 && !chaos) {
	       warning(1, "Your team's stuggling economy cannot support such an expenditure!", -1);
	       sendPickokPacket(0);
	       teamPick = -1;
	       continue;
	    }
	    for (i = 0; i < MAXPLAYER; i++) {
	       if (i != me->p_no &&
		   players[i].p_status == PALIVE &&
		   players[i].p_team == (1 << teamPick) &&
		   players[i].p_ship.s_type == STARBASE && !chaos) {
		  warning(1, "Your side already has a starbase!", -1);
		  sendPickokPacket(0);
		  teamPick = -1;
		  break;
	       }
	    }
	 }
	 if (teamPick == -1)
	    continue;
	 break;
      }
   }

   if (me->p_team != (1 << teamPick)) {   
      me->p_hostile |= me->p_team;
   }

   *team = teamPick;
   *stype = shipPick;

#ifdef FASTER_SB_MAXUPDATES
   if(timerDelay <= MAXUPDATES)
      handleUpdatesReq(NULL);
#endif

   sendPickokPacket(1);
   flushSockBuf();
}

int             diagnolTeams[9] = {0, KLI, ORI, 0, FED, 0, 0, 0, ROM};
int             adjTeams[9][2] = {{0, 0}, /* FED */ {ROM, ORI}, /* ROM */ {KLI, FED},
{0, 0}, /* KLI */ {ROM, ORI}, {0, 0}, {0, 0}, {0, 0},
 /* ORI */ {KLI, FED}};		/* for completeness -- TSH */

int
tournamentMask(team)
   int             team;
{
   int             i, team1 = 0, team2 = 0, num1 = 0, num2 = 0;

   if (strcmp(nombreDios, me->p_name) == 0 || overload)
      return (ALLTEAM);
   if (mustexit)
      return (0);
   if (status->gameup == 0)
      return (0);
   if (gamedown)
      return 0;

   if (chaos)
      return (ALLTEAM);

#ifdef nodef
   if (team == ALLTEAM) {
      team = 0;
      if (realNumShips(FED) == 0)
	 team |= FED;
      if (realNumShips(ROM) == 0)
	 team |= ROM;
      if (realNumShips(ORI) == 0)
	 team |= ORI;
      if (realNumShips(KLI) == 0)
	 team |= KLI;
   }
   return team;
#endif

   if (team != ALLTEAM && deadTeam(team)) {
      team = ALLTEAM;
   }
   if (team != ALLTEAM && realNumShips(diagnolTeams[team]) >= tournplayers) {
      team = ALLTEAM;
   }
   if (!(status->tourn))
      return ALLTEAM;

   for (i = 0; i < NUMTEAM; i++) {
      if (!(team & (1 << i)))
	 continue;
      if (deadTeam(1 << i)) {	/* if team i is dead, lock it out */
	 team &= ~(1 << i);
      }
      /*
       * if the diagonal of team i has enough players for tmode, lock this
       * team out
       */
      if (!deadTeam(diagnolTeams[1 << i]) &&
	  realNumShips(diagnolTeams[1 << i]) >= tournplayers) {
	 team &= ~(1 << i);
      }
   }
   /* if first time, check for tournament rules */
   /* for (i = 0; i < tournplayers; i++)  */
   for (i = 0; i < NUMTEAM; i++) {
      if (deadTeam(1 << i))
	 continue;
      if (realNumShips(1 << i) >= tournplayers) {
	 if (!team1) {
	    team1 = (1 << i);
	 } else {
#ifdef NODIAG
	    if (diagnolTeams[1 << i] != team1 ||
	    /*
	     * there's only one situation where diagonal teams are allowed.
	     * this is if both teams on either side of team1 are dead.
	     */
	     (deadTeam(adjTeams[team1][0]) && deadTeam(adjTeams[team1][1])))
#endif
	       team2 = (1 << i);
	 }
	 if (team2) {
	    num1 = realNumShips(team1);
	    num2 = realNumShips(team2);
	    if (ABS(num1 - num2) < 1) {	/* allow only 1 diff -- TC */
	       if (team & (team1 | team2)) {
		  return (team & (team1 | team2));
	       }
	       return (team1 | team2);
	    } else if ((num1 > num2) && (team != team1)) {
	       return (team2);
	    } else if ((num1 < num2) && (team != team2)) {
	       return (team1);
	    } else {
	       if (ABS(num1 - num2) < 2 ||
		   ((num1 > num2 && team == team2) ||
		    (num2 > num1 && team == team1)))
		  return (team);
	       else
		  /* Give them choice if their own team is too strong */
		  return (team1 | team2);
	    }
	 }
      }
   }
   return (team);
}

int
deadTeam(owner)
   int             owner;
{
   int             i, num = 0;
   struct planet  *p;

   if (planets[remap[owner] * 10 - 10].pl_couptime == 0)
      return (0);
   for (i = 0, p = planets; i < MAXPLANETS; i++, p++) {
      if (p->pl_owner & owner) {
	 num++;
      }
   }
   if (num != 0)
      return (0);
   return (1);
}
