/*
 * input.c
 */
#include "copyright.h"

#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <errno.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "packets.h"
#ifndef _NO_DEPEND
#   include "ntserv_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern int		      shutdown P_((int s, int how));
extern int                    fprintf P_((FILE *, const char *, ...));
extern int                    fflush P_((FILE *));
extern int                    setitimer P_((int, struct itimerval *, struct itimerval *));
extern void                   bzero P_((void *, size_t));
extern unsigned int           sleep P_((unsigned int));
extern int		      select P_((int width, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout));
extern int                    printf P_((const char *, ...));
extern pid_t                  getpid P_((void));
extern int                    close P_((int));

#undef P_
#endif

#define OUTFITTIME      (6 * AUTOQUIT * 1000000 / UPDATE)	/* 6 * AQ secs */

static int      sendflag = 0;
static int      do_update = 1;

void
setflag()
{
   /*
   printf("SIGNAL at %d\n", mstime());
   */
   if (do_update)
      intrupt();
   else
      sendflag = 1;

   /* statements below are executed once per timerDelay */

   /*
    * note that these two statements were originally expected to be checked
    * once per 100000 (rather then once per timerDelay), so we duplicate the
    * check after each readFromClient, below
    */

   if (me->p_updates > delay) {
      me->p_flags &= ~(PFWAR);
   }
   if (me->p_updates > rdelay) {
      me->p_flags &= ~(PFREFITTING);
   }
   /* this will ghostbust a player if no ping has been received in 5 seconds */
   if (ping_allow_ghostbust && me->p_status == PALIVE && ping_ghostbust >
       ping_ghostbust_interval) {
      /* ghostbusted */
      me->p_ghostbuster = GHOSTTIME;
      ping_ghostbust = 0;
      fprintf(stderr,
	      "GHOSTBUSTED from ping: %c%c: %s@%s\n", me->p_mapchars[0],
	      me->p_mapchars[1],
	      login, host);
      fflush(stderr);
   } else
      /* Keep self from being nuked... */
      me->p_ghostbuster = 0;
}

void
input()
{
#ifndef DS
   struct itimerval udt;
#endif
   fd_set          readfds;

#ifndef DS
   udt.it_interval.tv_sec = 0;
   udt.it_interval.tv_usec = timerDelay;
   udt.it_value.tv_sec = 0;
   udt.it_value.tv_usec = timerDelay;
   setitimer(ITIMER_REAL, &udt, 0);
#endif
   signal(SIGALRM, setflag);

   /* Idea:  read from client often, send to client not so often */
   while (1) {
      if (isClientDead()) {
	 if (!reconnect()) {
	    /*
	     * speed this up -- no reason for daemon to wait for additional 6
	     * minutes
	     */
	    me->p_ghostbuster = OUTFITTIME;

	    exitGame(1);
	 }
      }
      FD_ZERO(&readfds);
      FD_SET(sock, &readfds);
      if (udpSock >= 0)
	 FD_SET(udpSock, &readfds);

      /*
       * blocks indefinitely unless client input or timerDelay SIGALRM causes
       * EINTR
       */

      if (select(32, &readfds, 0, 0, NULL) > 0) {

	 do_update = 0;		/* don't let updates interfere with read */
	 ping_ghostbust = 0;

	 if (me->p_updates > delay) {
	    me->p_flags &= ~(PFWAR);
	 }
	 if (me->p_updates > rdelay) {
	    me->p_flags &= ~(PFREFITTING);
	 }
	 readFromClient();
	 /* happens if a client read was interrupted */
	 if (sendflag) {
	    intrupt();
	    sendflag = 0;
	 }
	 do_update = 1;		/* ok now */
      }
   }
}

int
reconnect()
{
   int             i;
#ifndef DS
   struct itimerval udt;
#endif

   if (noressurect)
      exitGame(0);
#ifndef DS
   udt.it_interval.tv_sec = 0;
   udt.it_interval.tv_usec = 0;
   udt.it_value.tv_sec = 0;
   udt.it_value.tv_usec = 0;
   setitimer(ITIMER_REAL, &udt, 0);
#endif
   signal(SIGALRM, SIG_IGN);
   signal(SIGIO, SIG_IGN);
#ifdef DEBUG
   printf("Ack!  The client went away!\n");
   printf("I will attempt to resurrect him!\n");
   fflush(stdout);
#endif
   commMode = COMM_TCP;
   if (udpSock >= 0)
      closeUdpConn();

printf("%s@%s ghostbusted.\n",me->p_login,me->p_monitor);       /* DEBUG */
fflush(stdout);

   /* For next two minutes, we try to restore connection */
   shutdown(sock, 2);
   sock = -1;
   for (i = 0;; i++) {
      me->p_ghostbuster = 0;
      if (connectToClient(host, nextSocket))
	 break;
      my_sleep(5);
      if (i == 23) {
#ifdef DEBUG
	 printf("Oh well, maybe I'm getting rusty!\n");
	 fflush(stdout);
#endif
printf("%s@%s: can't connect.\n",me->p_login,me->p_monitor);    /* DEBUG */
fflush(stdout);
	 return 0;
      }
   }
#ifndef DS
   udt.it_interval.tv_sec = 0;
   udt.it_interval.tv_usec = timerDelay;
   udt.it_value.tv_sec = 0;
   udt.it_value.tv_usec = timerDelay;
   setitimer(ITIMER_REAL, &udt, 0);
#endif
   signal(SIGALRM, setflag);
#ifdef DEBUG
   printf("A miracle!  He's alive!\n");
   fflush(stdout);
#endif
#ifdef DS
   if(me->p_process != getpid()){
      /* whoops, co-pilot.  We're outa here */
      forceShutdown(-11);
   }
#endif

   return 1;
}
