/*
 * plasma.c
 */
#include "copyright.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "packets.h"
#ifndef _NO_DEPEND
#   include "ntserv_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

#undef P_
#endif

/* Launch torp */

/*
 * * If a set of given conditions are met, fire a single torp in direction *
 * course.  Type is used because robots are allowed to shoot straight. *
 * Torps sent with status TMOVE wobble a bit.  TSTRAIGHT torps (fired * by
 * robots) move in a straight line. *
 * 
 * torp->t_fuse is the life of the torpedo.  It is set here based on * a
 * random function.  Torps currently live two to five seconds.
 */

void
nplasmatorp(course, type)
   unsigned char   course;
   int             type;
{
   register int    i;
   register struct plasmatorp *k;

   if (weaponsallowed[WP_PLASMA] == 0) {
      warning(1, "Plasmas haven't been invented yet.", 17);
      return;
   }
   if (me->p_ship.s_plasmacost < 0) {
      warning(1, "Weapon's Officer:  Captain, this ship is not armed with plasma torpedoes!", 18);
      return;
   }
   if (me->p_flags & PFWEP) {
      warning(1, "Plasma torpedo launch tube has exceeded the maximum safe temperature!", 19);
      return;
   }
   if ((me->p_nplasmatorp == MAXPLASMA) && (me->p_ship.s_type != ATT)) {
      warning(1, "Our fire control system limits us to 1 live torpedo at a time captain!", 20);
      return;
   } else if (me->p_nplasmatorp == MAXPLASMA) {
      warning(1, "Our fire control system limits us to 1 live torpedo at a time captain!", 20);
      return;
   }
   if (me->p_fuel < myship->s_plasmacost) {
      warning(0, "We don't have enough fuel to fire a plasma torpedo!", 22);
      return;
   }
   if (me->p_flags & PFREPAIR) {
      warning(1, "We cannot fire while our vessel is undergoing repairs.", 23);
      return;
   }
   if ((me->p_cloakphase > (CLOAK_PHASES - 1) - cloakdelay) &&
       (me->p_ship.s_type != ATT)) {
      warning(0, "We are unable to fire while in cloak, captain!", 24);
      return;
   }
   me->p_nplasmatorp++;
   me->p_fuel -= myship->s_plasmacost;
   me->p_wtemp += (myship->s_plasmacost / 10) - 8;	/* Heat weapons */
   for (i = me->p_no * MAXPLASMA, k = &plasmatorps[i];	/* Find a free plasma
							 * torp */
	i < me->p_no * MAXPLASMA + MAXPLASMA; i++, k++) {
      if (k->pt_status == PTFREE)
	 break;
   }

   /* Setup data in new torp */
   k->pt_no = i;
   k->pt_status = type;
   k->pt_owner = me->p_no;
   k->pt_team = me->p_team;
   k->pt_x = me->p_x;
   k->pt_y = me->p_y;
   if ((myship->s_type == STARBASE) || (myship->s_type == ATT))
      k->pt_dir = course;
   else
      k->pt_dir = me->p_dir;
   k->pt_damage = myship->s_plasmadamage;
   k->pt_speed = myship->s_plasmaspeed;
   k->pt_war = me->p_hostile | me->p_swar;
   k->pt_fuse = myship->s_plasmafuse;
   k->pt_turns = myship->s_plasmaturns;
}
